from unittest import TestCase
from src import module_helpers
import json


class TestRequestsHelpers(TestCase):

    h = module_helpers.RequestsHelpers()

    def test_request_json(self):
        msg, val = self.h.request_json(
            'http://echo.jsontest.com/key1/key-val/key2/key-val2', return_code=200)
        if not val:
            self.fail()
        j = json.loads(msg)
        self.assertDictEqual({'key2': 'key-val2', 'key1': 'key-val'}, j)

    def test_request_content(self):
        msg, val = self.h.request_content('https://httpbin.org/ip')
        if not val:
            self.fail()
        j = json.loads(msg)

    def test_request_raw(self):
        r, val = self.h.request_raw('https://httpbin.org/ip')
        if not val:
            self.fail()
        j = json.loads(r.content)



#!/usr/bin/python3.6
import argparse
import json
import logging
import os
import sys

from simplydomain.src import core_printer
from simplydomain.src import core_runtime

from simplydomain.src import module_resolvers
from simplydomain.src import core_logger

# import json config..
from simplydomain import config


def _raw_depth_check(value):
    ivalue = int(value)
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(
            "%s is an invalid positive int value" % value)
    if ivalue >= 6:
        raise argparse.ArgumentTypeError(
            "%s is too large of a keyspace for raw depth" % value)
    return ivalue


def cli_parse():
    """
    Parse the CLI args passed to the script.
    :return: args
    """
    # required
    parser = argparse.ArgumentParser()
    parser.add_argument("DOMAIN", help="domain to query")
    # opts
    parser.add_argument("-wb", "--wordlist-bruteforce", help="enable word list bruteforce module",
                        action="store_true")
    parser.add_argument("-wc", "--wordlist-count", help="set the count of the top words to use DEFAULT: 100",
                        action="store", default=100, type=int)
    parser.add_argument("-rb", "--raw-bruteforce", help="enable raw bruteforce module",
                        action="store_true")
    parser.add_argument("-rd", "--raw-depth", help="set the count of depth to raw bruteforce DEFAULT: 3",
                        action="store", default=3, type=_raw_depth_check)
    parser.add_argument("-m", "--module", help="module to hit",
                        action="store")
    parser.add_argument(
        "-o", "--output", help="output directory location (Ex. /users/test/)")
    parser.add_argument("-on", "--output-name",
                        help="output directory name (Ex. test-2017)",)
    parser.add_argument("-l", "--list", help="list loaded modules",
                        action="store_true")
    parser.add_argument("-ll", "--long-list", help="list loaded modules and info about each module",
                        action="store_true")
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    parser.add_argument("-d", "--debug", help="enable debug logging to .SimplyDns.log file, default WARNING only",
                        action="store_true")
    args = parser.parse_args()
    if args.verbose:
        print("[!] verbosity turned on")
    return args, parser


def load_config(pr):
    """
    Loads .config.json file for use
    :return: dict obj
    """
    json_file = config.__json_config
    ds = module_resolvers.DnsServers()
    ds.populate_servers()
    json_file = ds.populate_config(json_file)
    print(pr.blue_text('Public DNS resolvers populated: (SERVER COUNT: %s)' %
                       (str(ds.count_resolvers()))))
    return json_file


def main():
    """
    Print entry screen and pass execution to CLI, 
    and task core.
    :return: 
    """
    pr = core_printer.CorePrinters()
    pr.print_entry()
    args, parser = cli_parse()
    logger = core_logger.CoreLogging()
    pr.print_config_start()
    config = load_config(pr)
    config['args'] = args
    if args.debug:
        pr.print_green_on_bold('[!] DEBUGGING ENABLED!')
        logger.start(logging.DEBUG)
    else:
        logger.start(logging.INFO)
    logger.infomsg('main', 'startup')
    if args.module:
        if not args.DOMAIN:
            parser.print_help()
        config['silent'] = False
        c = core_runtime.CoreRuntime(logger, config)
        c.execute_mp()
    elif args.list:
        c = core_runtime.CoreRuntime(logger, config)
        c.list_modules()
    elif args.long_list:
        c = core_runtime.CoreRuntime(logger, config)
        c.list_modules_long()
    else:
        if not args.DOMAIN:
            parser.print_help()
        config['silent'] = False
        c = core_runtime.CoreRuntime(logger, config)
        c.execute_mp()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print('Interrupted')
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)



from fake_useragent import UserAgent
import json

from . import module_helpers


class DnsServers(module_helpers.RequestsHelpers):

    """
    A set of functions to find resolvers to use
    of high quality.
    """

    def __init__(self):
        """
        Init class structure.
        """
        module_helpers.RequestsHelpers.__init__(self)
        self.ua = UserAgent()
        self.nameservers = []
        self.nameserver_ips = []

    def populate_servers(self):
        """
        Populate server list.
        :return: NONE
        """
        data, status = self.request_json(
            'https://public-dns.info/nameserver/us.json')
        if status:
            data = json.loads(data)
            for d in data:
                self.nameservers.append(d)
        self.clean_servers()

    def populate_config(self, json_config):
        """
        Populate the json config file at runtime with
        public dns servers.
        :param json_config: start JSON config
        :return: json_config: final JSON config
        """
        json_config['resolvers'] = self.nameserver_ips[0:10]
        return json_config

    def clean_servers(self):
        """
        Sort name servers.
        :return: NONE
        """
        for i in self.nameservers:
            # check for 100% reliability
            if i['reliability'] == 1:
                self.nameserver_ips.append(i['ip'])

    def count_resolvers(self):
        """
        Count resolvers.
        :return: INT nameserver list count
        """
        return len(self.nameserver_ips)



import json
import time

from simplydomain.src import core_serialization
from simplydomain.src import module_helpers

from simplydomain.src import core_scrub


# use RequestsHelpers() class to make requests to target URL
class DynamicModule(module_helpers.RequestsHelpers):
    """
    Dynamic module class that will be loaded and called 
    at runtime. This will allow modules to easily be independent of the 
    core runtime.
    """

    def __init__(self, json_entry):
        """
        Init class structure. Each module takes a JSON entry object which 
        can pass different values to the module with out changing up the API.
        adapted form  Empire Project:
        https://github.com/EmpireProject/Empire/blob/master/lib/modules/python_template.py

        :param json_entry: JSON data object passed to the module.
        """
        module_helpers.RequestsHelpers.__init__(self)
        self.json_entry = json_entry
        self.info = {
            # mod name
            'Module': 'virus_total.py',

            # long name of the module to be used
            'Name': 'Virus Total Subdomain Search',

            # version of the module to be used
            'Version': '1.0',

            # description
            'Description': ['Uses https://virustotal.com search',
                            'with unofficial search engine API support.'],

            # authors or sources to be quoted
            'Authors': ['@Killswitch-GUI'],

            # list of resources or comments
            'comments': [
                'Searches for seen subdomains at one point and time.'
            ]
        }

        self.options = {

            'url': 'https://www.virustotal.com/ui/domains/%s/subdomains'
        }

    def dynamic_main(self, queue_dict):
        """
        Main entry point for process to call.

        core_serialization.SubDomain Attributes:
            name: long name of method
            module_name: name of the module that performed collection 
            source: source of the subdomain or resource of collection
            module_version: version from meta
            source: source of the collection
            time: time the result obj was built
            subdomain: subdomain to use
            valid: is domain valid

        :return: NONE
        """
        core_args = self.json_entry['args']
        task_output_queue = queue_dict['task_output_queue']
        cs = core_scrub.Scrub()
        domain = self.options['url'] % (str(core_args.DOMAIN))
        data, status = self.request_json(domain)
        data = json.loads(data)
        if status:
            for d in data['data']:
                cs.subdomain = d['id']
                # check if domain name is valid
                valid = cs.validate_domain()
                # build the SubDomain Object to pass
                sub_obj = core_serialization.SubDomain(
                    self.info["Name"],
                    self.info["Module"],
                    self.options['url'],
                    self.info["Version"],
                    time.time(),
                    d['id'],
                    valid
                )
                # populate queue with return data object
                task_output_queue.put(sub_obj)



import logging


class CoreLogging(object):
    """simple logging testing and dev"""

    def __init__(self):
        self.name = ".simplydomain.log"

    def start(self, level=logging.INFO):
        logger = logging.getLogger("simplydomain")
        logger.setLevel(level)
        fh = logging.FileHandler(self.name)
        formatter = logging.Formatter(
            '%(asctime)s-[%(name)s]-[%(levelname)s]- %(message)s')
        fh.setFormatter(formatter)
        logger.addHandler(fh)
        logger.info("Program started")
        logging.captureWarnings(True)
        logger.info("Set Logging Warning Capture: True")

    def debugmsg(self, message, modulename):
        try:
            msg = 'simplydomain.' + str(modulename)
            logger = logging.getLogger(msg)
            logger.debug(str(message))
        except Exception as e:
            print(e)

    def infomsg(self, message, modulename):
        try:
            msg = 'simplydomain.' + str(modulename)
            logger = logging.getLogger(msg)
            logger.info(str(message))
        except Exception as e:
            print(e)

    def warningmsg(self, message, modulename):
        try:
            msg = 'simplydomain.' + str(modulename)
            logger = logging.getLogger(msg)
            logger.warning(str(message))
        except Exception as e:
            print(e)



import multiprocessing as mp


class ModuleRecursion(object):
	"""Class to handle recursion.
	
	Simple class to handle tracking and storing prior 
	sub-domains discovred.
	""" 

	def __init__(self):
		"""class init.
		""" 
		self.recursion_queue = mp.Queue()

	def add_subdomain(self, domain):
		"""add subdomain to Q.
		
		uses a non-blocking call to add to the Q
		to prevent any errors with size. 
		
		Arguments:
			domain {str} -- subdomain to add to Q
		"""  
		self.recursion_queue.put(domain)

	def get_subdomain_list(self, valid_only=True):
		"""build subdomain list.
		
		Using the JSON from the event consumer, we
		can easily build a unique list of 
		subdomains for  module use.
		
		Keyword Arguments:
			valid_only {bool} -- filter only valid subdomains (default: {True})
		
		Returns:
			list -- list of raw subdomains
		"""
		data = []
		refill = []
		while True: 
			try:
				x = self.recursion_queue.get_nowait()
				if valid_only and x.valid:
					data.append(x.subdomain)
				if not valid_only:
					data.append(x.subdomain)
			except Exception as e:
				print(e)
				break
		return set(data)


from validators import domain


class Scrub(object):

    """
    Core data handler to clean, and post results in proper
    DataSerialization format for SimplyDomain.

    Attributes:
        subdomain: subdomain to parse
    """

    def __init__(self, subdomain=""):
        """
        Init class struc. Used as a object to parse and populate
        results.
        """
        self.subdomain = subdomain

    def validate_domain(self):
        """
        Use domain validator to confirm domain name. 
        :return: BOOL
        """
        try:
            val = domain(str(self.subdomain))
            if val:
                # domain is valid
                return True
            else:
                # domain validation failed
                return False
        except Exception as e:
            # TODO: add in logger class for errors
            return False



import sys
import time

from . import core_output
from . import core_processes
from . import module_loader


class CoreRuntime(module_loader.LoadModules,
                  core_processes.CoreProcess,
                  core_output.CoreOutput):
    """
    Core Runtime Class.
    """

    def __init__(self, logger, config):
        """
        Init class and passed objects.
        """
        self.config = config
        core_output.CoreOutput.__init__(self)
        module_loader.LoadModules.__init__(self)
        # ore_printer.CorePrinters.__init__(self)
        core_processes.CoreProcess.__init__(self)
        self.logger = logger

    def list_modules(self):
        """
        List the modules loaded.
        :return: 
        """
        self.logger.infomsg('tasked to list modules', 'CoreRuntime')
        self.print_modules(self.modules)

    def list_modules_long(self):
        """
        List the modules loaded.
        :return: 
        """
        self.logger.infomsg('tasked to list modules', 'CoreRuntime')
        self.print_modules_long(self.modules)

    def execute_output(self):
        """
        Execute the output of formatted data stucs.
        :return: NONE
        """
        self.logger.infomsg('starting execute_output()', 'CoreRuntime')
        self.output_text(self.serialize_json_output)
        self.output_json(self.serialize_json_output)
        self.output_text_std(self.serialize_json_output)

    def execute_output_object(self, return_type):
        """
        Execute the output of formatted data stucs.
        :return: NONE
        """
        self.logger.infomsg('starting execute_output_object()', 'CoreRuntime')
        if return_type.lower() == 'dict':
            return self.serialize_json_output
        else:
            return self.output_json_obj(self.serialize_json_output)

    def execute_startup(self):
        """
        setup Q.
        :return: NONE
        """
        self.logger.infomsg('execute_startup() setup began', 'CoreRuntime')
        self.print_d_module_start()
        self.logger.infomsg(
            'execute_startup() start to populate task queue', 'CoreRuntime')
        self.populate_task_queue(self.modules)
        self.logger.infomsg(
            'execute_startup() start to create hollow processes for future work', 'CoreRuntime')
        self.start_processes()
        self.config['silent'] = False

    def execute_dynamic(self):
        """
        Execute only the dynamic modules.
        :return: NONE
        """
        self.logger.infomsg(
            'execute_dynamic() start ONLY dynamic modules', 'CoreRuntime')
        self._start_thread_function(self._pbar_thread)
        while self.check_active():
            try:
                self.logger.infomsg(
                    'execute_dynamic() checking for active PIDs', 'CoreRuntime')
                time.sleep(2)
            except KeyboardInterrupt:
                self.logger.warningmsg(
                    'execute_dynamic() CRITICAL: CTRL+C Captured', 'CoreRuntime')
                self.print_red_on_bold("\n[!] CRITICAL: CTRL+C Captured - Trying to clean up!\n"
                                       "[!] WARNING: Press CTRL+C AGAIN to bypass and MANUALLY cleanup")
                try:
                    time.sleep(0.1)
                    self.stop_threads()
                    self.kill_processes()
                    sys.exit(0)
                except KeyboardInterrupt:
                    self.list_processes()
                    sys.exit(0)
        # cleanup dynamic mod pbar
        self.logger.infomsg(
            'execute_dynamic() dynamic modules completed', 'CoreRuntime')
        self.progress_bar_pickup.put(None)
        self.close_progress_bar()

    def execute_static(self):
        """
        Execute static modules in sorted order by EXEC order.
        :return: NONE
        """
        self.logger.infomsg(
            'execute_static() start ONLY static modules', 'CoreRuntime')
        self.print_s_module_start()
        queue_dict = {
            'task_queue': self.task_queue,
            'task_output_queue': self.task_output_queue,
            'subdomain_list': self.get_subdomain_list(),
        }
        if self.config['args'].wordlist_bruteforce:
            self.execute_process(
                'simplydomain/src/static_modules/subdomain_bruteforce.py', self.config, queue_dict)
        if self.config['args'].raw_bruteforce:
            self.execute_process(
                'simplydomain/src/static_modules/subdomain_raw_bruteforce.py', self.config, queue_dict)
        self.logger.infomsg(
            'execute_static() static modules completed', 'CoreRuntime')

    def execute_bruteforce(self, return_type='json'):
        """
        Executes only the domain Bruteforce module and retunr data.
        """
        try:
            self.config['silent'] = True
            self.populate_task_queue(self.modules)
            self._start_threads()
            self._start_thread_function(self._pbar_thread)
            queue_dict = {
                'task_queue': self.task_queue,
                'task_output_queue': self.task_output_queue,
                'subdomain_list': self.get_subdomain_list(),
            }
            if self.config['args'].wordlist_bruteforce:
                self.execute_process(
                    'simplydomain/src/static_modules/subdomain_bruteforce.py', self.config, queue_dict)
        finally:
            self.join_threads()
        # return JSON object
        return self.execute_output_object(return_type)

    def execute_raw_bruteforce(self, return_type='json'):
        """
        Executes only the domain Bruteforce module and retunr data.
        """
        try:
            self.config['silent'] = True
            self.populate_task_queue(self.modules)
            self._start_threads()
            self._start_thread_function(self._pbar_thread)
            queue_dict = {
                'task_queue': self.task_queue,
                'task_output_queue': self.task_output_queue,
            }
            if self.config['args'].raw_bruteforce:
                self.execute_process(
                    'simplydomain/src/static_modules/subdomain_raw_bruteforce.py', self.config, queue_dict)
        finally:
            self.join_threads()
        # return JSON object
        return self.execute_output_object(return_type)

    def execute_amp(self, return_type='json'):
        """
        Executes all the Dynamic Modules to be 
        sent to processes. Returns raw object.
        """
        try:
            # startup
            self.execute_startup()
            self.execute_dynamic()
            self.execute_static()
        finally:
            # cleanup
            self.join_processes()
            self.join_threads()
        # return JSON object
        return self.execute_output_object(return_type)

    def execute_mp(self):
        """
        Executes all the Dynamic Modules to be 
        sent to processes
        :return: 
        """
        try:
            # startup
            self.execute_startup()
            self.execute_dynamic()
            self.execute_static()
        finally:
            # cleanup
            self.join_processes()
            self.join_threads()
            self.execute_output()



import json
import os
import pathlib
import time

from . import core_printer


class CoreOutput(core_printer.CorePrinters):
    """
    Core class to handle final output.
    """

    def __init__(self):
        """
        Init class.
        """
        core_printer.CorePrinters.__init__(self)
        self.json_data = {}

    def output_json_obj(self, json_data):
        """
        Output json data file.
        :param json_data: json obj
        :return: NONE
        """
        return json.dumps(json_data.subdomains, sort_keys=True)

    def output_json(self, json_data):
        """
        Output json data file.
        :param json_data: json obj
        :return: NONE
        """
        args = self.config['args']
        s = str(args.DOMAIN)
        s = s.replace('.', '-')
        loc = ""
        dir_name = s + '-' + str(int(time.time()))
        def_name = s + '.json'
        if args.output:
            loc += str(args.output)
        if args.output_name:
            dir_name = str(args.output_name)
        dir_to_write = os.path.join(loc, dir_name)
        pathlib.Path(dir_to_write).mkdir(parents=True, exist_ok=True)
        with open(os.path.join(dir_to_write, def_name), 'a') as outfile:
            json.dump(json_data.subdomains, outfile, sort_keys=True, indent=4)
        self.logger.infomsg('output_json() JSON output file created at: : '
                            + str((os.path.join(dir_to_write, def_name))), 'CoreOutput')
        print(self.blue_text("JSON text file created: %s" %
                             (os.path.join(dir_to_write, def_name))))

    def print_text(self, json_data):
        """
        Output to text file
        :param json_data: json obj
        :return: NONE
        """
        for item in json_data.subdomains['data']:
            print("name:%s module_name:%s module_version:%s source:%s time:%s toolname:%s subdomain:%s vaild:%s" %
                  (item['name'], item['module_name'], item['module_version'], item['source'], item['time'],
                   item['toolname'], item['subdomain'], item['valid']))

    def output_text(self, json_data):
        """
        Output to text file
        :param json_data: json obj
        :return: NONE
        """
        args = self.config['args']
        s = str(args.DOMAIN)
        s = s.replace('.', '-')
        loc = ""
        dir_name = s + '-' + str(int(time.time()))
        def_name = s + '.grep'
        if args.output:
            loc += str(args.output)
        if args.output_name:
            dir_name = str(args.output_name)
        dir_to_write = os.path.join(loc, dir_name)
        pathlib.Path(dir_to_write).mkdir(parents=True, exist_ok=True)
        with open(os.path.join(dir_to_write, def_name), 'a') as outfile:
            for item in json_data.subdomains['data']:
                x = ("name:%s module_name:%s module_version:%s source:%s time:%s toolname:%s subdomain:%s vaild:%s\n" %
                     (item['name'], item['module_name'], item['module_version'], item['source'], item['time'],
                      item['toolname'], item['subdomain'], item['valid']))
                outfile.write(x)
        self.logger.infomsg('output_text() TXT grep output file created at: : '
                            + str(os.path.join(dir_to_write, def_name)), 'CoreOutput')
        print(self.blue_text("Grepable text file created: %s" %
                             (os.path.join(dir_to_write, def_name))))

    def output_text_std(self, json_data):
        """
        Output to text file
        :param json_data: json obj
        :return: NONE
        """
        args = self.config['args']
        s = str(args.DOMAIN)
        s = s.replace('.', '-')
        loc = ""
        dir_name = s + '-' + str(int(time.time()))
        def_name = s + '.txt'
        if args.output:
            loc += str(args.output)
        if args.output_name:
            dir_name = str(args.output_name)
        dir_to_write = os.path.join(loc, dir_name)
        pathlib.Path(dir_to_write).mkdir(parents=True, exist_ok=True)
        flist = []
        with open(os.path.join(dir_to_write, def_name), 'a') as outfile:
            for item in json_data.subdomains['data']:
                flist.append(item['subdomain'])
            for item in sorted(set(flist)):
                x = ("%s\n" % (item))
                outfile.write(x)
        self.logger.infomsg('output_text_std() TXT output file created at: : '
                            + str((os.path.join(dir_to_write, def_name))), 'CoreOutput')
        print(self.blue_text("Standard text file created: %s" %
                             (os.path.join(dir_to_write, def_name))))



import time

from dnsdumpster.DNSDumpsterAPI import DNSDumpsterAPI
from simplydomain.src import core_serialization
from simplydomain.src import module_helpers

from simplydomain.src import core_scrub


# use RequestsHelpers() class to make requests to target URL
class DynamicModule(module_helpers.RequestsHelpers):
    """
    Dynamic module class that will be loaded and called 
    at runtime. This will allow modules to easily be independent of the 
    core runtime.
    """

    def __init__(self, json_entry):
        """
        Init class structure. Each module takes a JSON entry object which 
        can pass different values to the module with out changing up the API.
        adapted form  Empire Project:
        https://github.com/EmpireProject/Empire/blob/master/lib/modules/python_template.py

        :param json_entry: JSON data object passed to the module.
        """
        module_helpers.RequestsHelpers.__init__(self)
        self.json_entry = json_entry
        self.info = {
            # mod name
            'Module': 'dnsdumpster_search.py',

            # long name of the module to be used
            'Name': 'Python API for Dnsdumpster',

            # version of the module to be used
            'Version': '1.0',

            # description
            'Description': ['(Unofficial) Python API for',
                            'https://dnsdumpster.com/ using @paulsec lib.'],

            # authors or sources to be quoted
            'Authors': ['@Killswitch-GUI', '@PaulSec'],

            # list of resources or comments
            'comments': [
                'Searches for prior seen domains, as well as data that goes with those.'
            ]
        }

        self.options = {

            'threads': '',
            'url': 'https://dnsdumpster.com',
        }

    def dynamic_main(self, queue_dict):
        """
        Main entry point for process to call.

        core_serialization.SubDomain Attributes:
            name: long name of method
            module_name: name of the module that performed collection 
            source: source of the subdomain or resource of collection
            module_version: version from meta
            source: source of the collection
            time: time the result obj was built
            subdomain: subdomain to use
            valid: is domain valid

        :return: NONE
        """
        core_args = self.json_entry['args']
        task_output_queue = queue_dict['task_output_queue']
        cs = core_scrub.Scrub()

        data = DNSDumpsterAPI().search(str(core_args.DOMAIN))
        for d in data['dns_records']['host']:
            cs.subdomain = d['domain']
            # check if domain name is valid
            valid = cs.validate_domain()
            # build the SubDomain Object to pass
            sub_obj = core_serialization.SubDomain(
                self.info["Name"],
                self.info["Module"],
                self.options['url'],
                self.info["Version"],
                time.time(),
                d['domain'],
                valid
            )
            # populate queue with return data object
            task_output_queue.put(sub_obj)
